# Generated by Django 3.2.5 on 2021-10-23 23:05

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReleaseInfo',
            fields=[
                ('Id', models.AutoField(primary_key=True, serialize=False)),
                ('AnnouncementSourceName', models.TextField(blank=True, default='')),
                ('AnnouncementId', models.TextField(blank=True, default='')),
                ('ReleaseName', models.TextField(blank=True, default='')),
                ('Type', models.TextField(blank=True, default='Feature Film')),
                ('ImdbId', models.TextField(blank=True, default='')),
                ('Directors', models.TextField(blank=True, default='')),
                ('Title', models.TextField(blank=True, default='')),
                ('Year', models.TextField(blank=True, default='')),
                ('Tags', models.TextField(blank=True, default='')),
                ('MovieDescription', models.TextField(blank=True, default='')),
                ('CoverArtUrl', models.TextField(blank=True, default='')),
                ('YouTubeId', models.TextField(blank=True, default='')),
                ('MetacriticUrl', models.TextField(blank=True, default='')),
                ('RottenTomatoesUrl', models.TextField(blank=True, default='')),
                ('Codec', models.TextField(blank=True, default='')),
                ('CodecOther', models.TextField(blank=True, default='')),
                ('Container', models.TextField(blank=True, default='')),
                ('ContainerOther', models.TextField(blank=True, default='')),
                ('ResolutionType', models.TextField(blank=True, default='')),
                ('Resolution', models.TextField(blank=True, default='')),
                ('Source', models.TextField(blank=True, default='')),
                ('SourceOther', models.TextField(blank=True, default='')),
                ('RemasterTitle', models.TextField(blank=True, default='')),
                ('RemasterYear', models.TextField(blank=True, default='')),
                ('JobStartMode', models.IntegerField(default=0)),
                ('JobRunningState', models.IntegerField(default=0)),
                ('FinishedJobPhase', models.IntegerField(default=0)),
                ('Flags', models.IntegerField(default=0)),
                ('ErrorMessage', models.TextField(blank=True, default='')),
                ('PtpId', models.TextField(blank=True, default='')),
                ('PtpTorrentId', models.TextField(blank=True, default='')),
                ('InternationalTitle', models.TextField(blank=True, default='')),
                ('Nfo', models.TextField(blank=True, default='')),
                ('SourceTorrentFilePath', models.TextField(blank=True, default='')),
                ('SourceTorrentInfoHash', models.TextField(blank=True, default='')),
                ('UploadTorrentCreatePath', models.TextField(blank=True, default='')),
                ('UploadTorrentFilePath', models.TextField(blank=True, default='')),
                ('UploadTorrentInfoHash', models.TextField(blank=True, default='')),
                ('ReleaseDownloadPath', models.TextField(blank=True, default='')),
                ('ReleaseUploadPath', models.TextField(blank=True, default='')),
                ('ReleaseNotes', models.TextField(blank=True, default='')),
                ('Screenshots', models.TextField(blank=True, default='')),
                ('LastModificationTime', models.DateTimeField(auto_now=True)),
                ('Size', models.IntegerField(default=0)),
                ('Subtitles', models.TextField(blank=True, default='')),
                ('IncludedFiles', models.TextField(blank=True, default='')),
                ('DuplicateCheckCanIgnore', models.IntegerField(default=0)),
                ('ScheduleTimeUtc', models.DateTimeField()),
            ],
        ),
    ]
