# Generated by Django 3.2.5 on 2021-10-25 03:20

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='releaseinfo',
            name='Trumpable',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='releaseinfo',
            name='ScheduleTimeUtc',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
