@keyframes spinner {
  to { transform: rotate(360deg); }
}

div#tree {
    height: 500px;
}

.jconfirm-content-pane ul.fancytree-container {
    border: 0;
    outline: 0;
}


.download-spinner::before {
	content: '';
	box-sizing: border-box;
	position: relative;
	/* top: 50%; */
	/* left: 50%; */
	width: 16px;
	height: 16px;
	margin-top: 2px;
	margin-left: 9px;
	border-radius: 50%;
	border: 2px solid #48c774;
	border-top-color: #0000;
	animation: spinner 1.8s linear infinite;
	display: inline-block;
	margin-right: 4px;
}

.inprogress-spinner::before {
	content: '';
	box-sizing: border-box;
	position: relative;
	/* top: 50%; */
	/* left: 50%; */
	width: 16px;
	height: 16px;
	margin-top: 2px;
	margin-left: 9px;
	border-radius: 50%;
	border: 2px solid rgb(0, 209, 178);
	border-top-color: #0000;
	border-bottom-color: #0000;
	animation: spinner 1.0s linear infinite;
	display: inline-block;
	margin-right: 4px;
}

table#jobs td.nowrap
{
    white-space: nowrap;
}

table#jobs td
{
    padding: 3px;
    font-size: 13px;
}

table#jobs span.job_title
{
    font-size: 14px;
}

html#uploader, body#uploader, table#jobs th, #jobs.table.is-striped tbody, td.label {
    font-family: Arial,sans-serif;
    background-color: #181818;
    color: #D1D4C9;
}

#editjob_table td.label
{
    text-align: right;
}

input[type="text"], input[type="password"], input[type="search"], textarea, button, select, textarea, span.select2, span.select2-selection--multiple.select2-selection {
    background-color : #363636;
    border: thin solid #30373d;
    border-radius: 3px;
    color: white;
}

#search_filter.input::placeholder {
    color: rgb(255, 255, 255);
}

span.select2-selection__choice__display, span.select2-dropdown, #uploader li.select2-selection__choice {
    background-color: #242424;
}

/* .select2-container--default li.select2-results__option--highlighted.select2-results__option--selectable { */
/*     background-color: #202020; */
/* } */

.select2-container--default li.select2-results__option--selected.select2-results__option--selectable {
    background-color: #303030;
}

table#editjob_table, table#jobs
{
    line-height: 1;
    border-collapse: collapse;
}

table#jobs
{
    line-height: 1.2;
    border-collapse: collapse;
}

#editjob_table td, th
{
    border: 1px;
    padding: 1px 3px 1px 3px;
    text-align: left;
}


body#uploader nav, body#uploader .navbar-item,  {
    background-color: #29435C;
    text: #161616;
}

table#jobs.table td, table#jobs.table th {
    border-color: #2f2e2e;
}

table#jobs.table.is-hoverable.is-striped tbody tr:not(.is-selected):hover {
    background-color: #252224;
}

table#jobs.table.is-striped tbody tr:not(.is-selected):nth-child(2n) {
    background-color: #101010;
}

table#jobs thead {
    border-bottom: 7px solid white;
}

table#jobs th::after, table#jobs th::before {
    color: #D1D4C9;
}

span.job_title, span.job_title a
{
    color: white;
    font-weight: bold;
}


table.jobs
{
    width: 100%;
}
