# Generated by Django 3.2.5 on 2021-10-27 19:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0002_auto_20211025_0320'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='JobRunningState',
            field=models.IntegerField(choices=[(0, 'Waiting for start'), (1, 'In progress'), (2, 'Paused'), (3, 'Finished'), (4, 'Failed'), (5, 'Ignored'), (6, 'Ignored, already exists'), (7, 'Ignored, forbidden'), (8, 'Ignored, missing info'), (9, 'Ignored, not supported'), (10, 'Downloaded, already exists'), (11, 'Scheduled'), (12, 'Downloading')], default=0),
        ),
    ]
