{% load static %}
<!doctype html>
<html id="uploader">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="{% static "favicon.png" %}">
    <link rel="stylesheet" type="text/css" href="{% static "normalize.css" %}">
    <link rel="stylesheet" type="text/css" href="{% static "style.css" %}">
    <script defer type="text/javascript" src="{% static "script/fontawesome.all.min.js" %}"></script>
    <link rel="stylesheet" type="text/css" href="{% static "script/bulma.min.css" %}"/>
    {% block head %}
    {% endblock %}
  </head>
  <body class="uploader" id="uploader">
    <header>
      <nav class="navbar is-dark" role="navigation" aria-label="main navigation">
        <div class="navbar-brand">
          &nbsp;
          <a class="navbar-item" href="/jobs">
            <span class="icon has-text-primary">P<i class="fas fa-arrow-up"></i>P</span>
          </a>
          <a role="button" class="navbar-burger" aria-label="menu" aria-expanded="false" data-target="navbarBasicExample">
            <span aria-hidden="true"></span>
            <span aria-hidden="true"></span>
            <span aria-hidden="true"></span>
          </a>
        </div>

        <div id="navbarBasicExample" class="navbar-menu">
          <div class="navbar-start">
	    <a class="navbar-item" href="{% url "jobs" %}">Jobs</a>
            <div class="navbar-item has-dropdown is-hoverable">
	      <a class="navbar-link" href="{% url "upload" %}">Upload</a>
              <div class="navbar-dropdown">
	        <a class="navbar-item" href="{% url "bulk_upload" %}">Bulk</a>
                <a class="navbar-item" href="{% url "upload" %}">Single</a>
              </div>
            </div>
	    <a class="navbar-item" href="{% url "settings" %}">Settings</a>
          </div>
        </div>
      </nav>
      <nav>
	<ul>
	</ul>
      </nav>
    </header>
    <!-- <section class="section"> -->
    <div class="container">
      {% block body %}
      {% endblock %}
    </div>
    <!-- </section> -->
  </body>
</html>
