# Generated by Django 3.2.5 on 2021-10-28 22:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0003_alter_releaseinfo_jobrunningstate'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='releaseinfo',
            name='Tags',
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='ForceDirectorylessSingleFileTorrent',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='OverrideScreenshots',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='PersonalRip',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='SceneRelease',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='SpecialRelease',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='StartImmediately',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='StopBeforeUploading',
            field=models.BooleanField(default=False),
        ),
    ]
