# Generated by Django 3.2.11 on 2022-01-15 03:38

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0014_rename_scheduletimeutc_releaseinfo_scheduletime'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='ScheduleTime',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=utc), null=True),
        ),
    ]
