# Generated by Django 3.2.5 on 2021-10-29 21:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0008_alter_releaseinfo_subtitles'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='Trumpable',
            field=models.JSONField(blank=True, default=list),
        ),
    ]
