# Generated by Django 3.2.5 on 2021-10-29 01:35

import re

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("web", "0004_auto_20211028_2254"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="releaseinfo",
            name="Flags",
        ),
        migrations.AddField(
            model_name="releaseinfo",
            name="Tags",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="releaseinfo",
            name="Subtitles",
            field=models.TextField(
                blank=True,
                default="",
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^\\d+(?:,\\d+)*\\Z"),
                        code="invalid",
                        message="Enter only digits separated by commas.",
                    )
                ],
            ),
        ),
    ]
