# Generated by Django 3.2.5 on 2021-10-31 19:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0010_alter_releaseinfo_screenshots'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='Screenshots',
            field=models.JSONField(blank=True, default={}),
        ),
    ]
