# Generated by Django 3.2.13 on 2022-06-03 16:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0015_alter_releaseinfo_scheduletime'),
    ]

    operations = [
        migrations.AddField(
            model_name='releaseinfo',
            name='BdInfo',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
